use weave::Error;

use posix_rs::PosixError;



/// The possible errors that may occur when
/// attempting to spawn a Daemon.
pub enum DaemonError
{
   /// There was an issue changing the working directory.
   ChangeDirectory
   {
      parent: PosixError
   },

   /// There was an issue closing a stream.
   CloseStream
   {
      parent: PosixError
   },

   /// There was an issue forking the current process.
   Fork
   {
      parent: PosixError
   },

   /// There was an issue creating a new session and
   /// making the current process group leader.
   SessionCreation
   {
      parent: PosixError
   }
}



impl ::std::fmt::Debug for DaemonError
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      ::std::fmt::Display::fmt(self, f)
   }
}

impl ::std::fmt::Display for DaemonError
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      write!(f, "{}", self.get_description())
   }
}

impl Error for DaemonError
{
   fn get_description(&self) -> &str
   {
      match *self
      {
         DaemonError::ChangeDirectory { parent: ref parent_error } =>
            { parent_error.get_description() }
         DaemonError::CloseStream { parent: ref parent_error } =>
            { parent_error.get_description() }
         DaemonError::Fork { parent: ref parent_error } =>
            { parent_error.get_description() }
         DaemonError::SessionCreation { parent: ref parent_error } =>
            { parent_error.get_description() }
      }
   }

   fn get_cause(&self) -> Option<&Error>
   {
      match *self
      {
         DaemonError::ChangeDirectory { parent: ref parent_error } =>
            { Some(parent_error) }
         DaemonError::CloseStream { parent: ref parent_error } =>
            { Some(parent_error) }
         DaemonError::Fork { parent: ref parent_error } =>
            { Some(parent_error) }
         DaemonError::SessionCreation { parent: ref parent_error } =>
            { Some(parent_error) }
      }
   }
}

